@@ -577,228 +577,162 @@ macro_rules! box_to_opt_ref {
     };
 }
 
+#[cfg(feature = "with-json")]
 impl Value {
     pub fn is_json(&self) -> bool {
-        #[cfg(feature = "with-json")]
         return matches!(self, Self::Json(_));
-        #[cfg(not(feature = "with-json"))]
-        return false;
     }
-    #[cfg(feature = "with-json")]
+
     pub fn as_ref_json(&self) -> Option<&Json> {
         match self {
             Self::Json(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::Json"),
         }
     }
-    #[cfg(not(feature = "with-json"))]
-    pub fn as_ref_json(&self) -> Option<&bool> {
-        panic!("not Value::Json")
-    }
 }
 
+#[cfg(feature = "with-chrono")]
 impl Value {
     pub fn is_chrono_date(&self) -> bool {
-        #[cfg(feature = "with-chrono")]
         return matches!(self, Self::ChronoDate(_));
-        #[cfg(not(feature = "with-chrono"))]
-        return false;
     }
-    #[cfg(feature = "with-chrono")]
+
     pub fn as_ref_chrono_date(&self) -> Option<&NaiveDate> {
         match self {
             Self::ChronoDate(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::ChronoDate"),
         }
     }
-    #[cfg(not(feature = "with-chrono"))]
-    pub fn as_ref_chrono_date(&self) -> Option<&bool> {
-        panic!("not Value::ChronoDate")
-    }
 }
 
+#[cfg(feature = "with-time")]
 impl Value {
     pub fn is_time_date(&self) -> bool {
-        #[cfg(feature = "with-time")]
         return matches!(self, Self::TimeDate(_));
-        #[cfg(not(feature = "with-time"))]
-        return false;
     }
-    #[cfg(feature = "with-time")]
+
     pub fn as_ref_time_date(&self) -> Option<&time::Date> {
         match self {
             Self::TimeDate(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::TimeDate"),
         }
     }
-    #[cfg(not(feature = "with-time"))]
-    pub fn as_ref_time_date(&self) -> Option<&bool> {
-        panic!("not Value::TimeDate")
-    }
 }
 
+#[cfg(feature = "with-chrono")]
 impl Value {
     pub fn is_chrono_time(&self) -> bool {
-        #[cfg(feature = "with-chrono")]
         return matches!(self, Self::ChronoTime(_));
-        #[cfg(not(feature = "with-chrono"))]
-        return false;
     }
-    #[cfg(feature = "with-chrono")]
+
     pub fn as_ref_chrono_time(&self) -> Option<&NaiveTime> {
         match self {
             Self::ChronoTime(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::ChronoTime"),
         }
     }
-    #[cfg(not(feature = "with-chrono"))]
-    pub fn as_ref_chrono_time(&self) -> Option<&bool> {
-        panic!("not Value::ChronoTime")
-    }
 }
 
+#[cfg(feature = "with-time")]
 impl Value {
     pub fn is_time_time(&self) -> bool {
-        #[cfg(feature = "with-time")]
         return matches!(self, Self::TimeTime(_));
-        #[cfg(not(feature = "with-time"))]
-        return false;
     }
-    #[cfg(feature = "with-time")]
+
     pub fn as_ref_time_time(&self) -> Option<&time::Time> {
         match self {
             Self::TimeTime(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::TimeTime"),
         }
     }
-    #[cfg(not(feature = "with-time"))]
-    pub fn as_ref_time_time(&self) -> Option<&bool> {
-        panic!("not Value::TimeTime")
-    }
 }
 
+#[cfg(feature = "with-chrono")]
 impl Value {
     pub fn is_chrono_date_time(&self) -> bool {
-        #[cfg(feature = "with-chrono")]
         return matches!(self, Self::ChronoDateTime(_));
-        #[cfg(not(feature = "with-chrono"))]
-        return false;
     }
-    #[cfg(feature = "with-chrono")]
+
     pub fn as_ref_chrono_date_time(&self) -> Option<&NaiveDateTime> {
         match self {
             Self::ChronoDateTime(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::ChronoDateTime"),
         }
     }
-    #[cfg(not(feature = "with-chrono"))]
-    pub fn as_ref_chrono_date_time(&self) -> Option<&bool> {
-        panic!("not Value::ChronoDateTime")
-    }
 }
 
+#[cfg(feature = "with-time")]
 impl Value {
     pub fn is_time_date_time(&self) -> bool {
-        #[cfg(feature = "with-time")]
         return matches!(self, Self::TimeDateTime(_));
-        #[cfg(not(feature = "with-time"))]
-        return false;
     }
-    #[cfg(feature = "with-time")]
+
     pub fn as_ref_time_date_time(&self) -> Option<&PrimitiveDateTime> {
         match self {
             Self::TimeDateTime(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::TimeDateTime"),
         }
     }
-    #[cfg(not(feature = "with-time"))]
-    pub fn as_ref_time_date_time(&self) -> Option<&bool> {
-        panic!("not Value::TimeDateTime")
-    }
 }
 
+#[cfg(feature = "with-chrono")]
 impl Value {
     pub fn is_chrono_date_time_utc(&self) -> bool {
-        #[cfg(feature = "with-chrono")]
         return matches!(self, Self::ChronoDateTimeUtc(_));
-        #[cfg(not(feature = "with-chrono"))]
-        return false;
     }
-    #[cfg(feature = "with-chrono")]
+
     pub fn as_ref_chrono_date_time_utc(&self) -> Option<&DateTime<Utc>> {
         match self {
             Self::ChronoDateTimeUtc(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::ChronoDateTimeUtc"),
         }
     }
-    #[cfg(not(feature = "with-chrono"))]
-    pub fn as_ref_chrono_date_time_utc(&self) -> Option<&bool> {
-        panic!("not Value::ChronoDateTimeUtc")
-    }
 }
 
+#[cfg(feature = "with-chrono")]
 impl Value {
     pub fn is_chrono_date_time_local(&self) -> bool {
-        #[cfg(feature = "with-chrono")]
         return matches!(self, Self::ChronoDateTimeLocal(_));
-        #[cfg(not(feature = "with-chrono"))]
-        return false;
     }
-    #[cfg(feature = "with-chrono")]
+
     pub fn as_ref_chrono_date_time_local(&self) -> Option<&DateTime<Local>> {
         match self {
             Self::ChronoDateTimeLocal(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::ChronoDateTimeLocal"),
         }
     }
-    #[cfg(not(feature = "with-chrono"))]
-    pub fn as_ref_chrono_date_time_local(&self) -> Option<&bool> {
-        panic!("not Value::ChronoDateTimeLocal")
-    }
 }
 
+#[cfg(feature = "with-chrono")]
 impl Value {
     pub fn is_chrono_date_time_with_time_zone(&self) -> bool {
-        #[cfg(feature = "with-chrono")]
         return matches!(self, Self::ChronoDateTimeWithTimeZone(_));
-        #[cfg(not(feature = "with-chrono"))]
-        return false;
     }
-    #[cfg(feature = "with-chrono")]
+
     pub fn as_ref_chrono_date_time_with_time_zone(&self) -> Option<&DateTime<FixedOffset>> {
         match self {
             Self::ChronoDateTimeWithTimeZone(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::ChronoDateTimeWithTimeZone"),
         }
     }
-    #[cfg(not(feature = "with-chrono"))]
-    pub fn as_ref_chrono_date_time_with_time_zone(&self) -> Option<&bool> {
-        panic!("not Value::ChronoDateTimeWithTimeZone")
-    }
 }
 
+#[cfg(feature = "with-time")]
 impl Value {
     pub fn is_time_date_time_with_time_zone(&self) -> bool {
-        #[cfg(feature = "with-time")]
         return matches!(self, Self::TimeDateTimeWithTimeZone(_));
-        #[cfg(not(feature = "with-time"))]
-        return false;
     }
-    #[cfg(feature = "with-time")]
+
     pub fn as_ref_time_date_time_with_time_zone(&self) -> Option<&OffsetDateTime> {
         match self {
             Self::TimeDateTimeWithTimeZone(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::TimeDateTimeWithTimeZone"),
         }
     }
-    #[cfg(not(feature = "with-time"))]
-    pub fn as_ref_time_date_time_with_time_zone(&self) -> Option<&bool> {
-        panic!("not Value::TimeDateTimeWithTimeZone")
-    }
 }
 
+#[cfg(feature = "with-chrono")]
 impl Value {
-    #[cfg(feature = "with-chrono")]
     pub fn chrono_as_naive_utc_in_string(&self) -> Option<String> {
         match self {
             Self::ChronoDate(v) => v.as_ref().map(|v| v.to_string()),
@@ -810,14 +744,10 @@ impl Value {
             _ => panic!("not chrono Value"),
         }
     }
-    #[cfg(not(feature = "with-chrono"))]
-    pub fn chrono_as_naive_utc_in_string(&self) -> Option<&bool> {
-        panic!("not chrono Value")
-    }
 }
 
+#[cfg(feature = "with-time")]
 impl Value {
-    #[cfg(feature = "with-time")]
     pub fn time_as_naive_utc_in_string(&self) -> Option<String> {
         match self {
             Self::TimeDate(v) => v.as_ref().map(|v| v.format("%Y-%m-%d")),
@@ -829,109 +759,67 @@ impl Value {
             _ => panic!("not time Value"),
         }
     }
-    #[cfg(not(feature = "with-time"))]
-    pub fn time_as_naive_utc_in_string(&self) -> Option<&bool> {
-        panic!("not time Value")
-    }
 }
 
+#[cfg(feature = "with-rust_decimal")]
 impl Value {
     pub fn is_decimal(&self) -> bool {
-        #[cfg(feature = "with-rust_decimal")]
         return matches!(self, Self::Decimal(_));
-        #[cfg(not(feature = "with-rust_decimal"))]
-        return false;
     }
-    #[cfg(feature = "with-rust_decimal")]
     pub fn as_ref_decimal(&self) -> Option<&Decimal> {
         match self {
             Self::Decimal(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::Decimal"),
         }
     }
-    #[cfg(feature = "with-rust_decimal")]
     pub fn decimal_to_f64(&self) -> Option<f64> {
         use rust_decimal::prelude::ToPrimitive;
         self.as_ref_decimal().map(|d| d.to_f64().unwrap())
     }
-    #[cfg(not(feature = "with-rust_decimal"))]
-    pub fn as_ref_decimal(&self) -> Option<&bool> {
-        panic!("not Value::Decimal")
-    }
-    #[cfg(not(feature = "with-rust_decimal"))]
-    pub fn decimal_to_f64(&self) -> Option<f64> {
-        None
-    }
 }
 
+#[cfg(feature = "with-bigdecimal")]
 impl Value {
     pub fn is_big_decimal(&self) -> bool {
-        #[cfg(feature = "with-bigdecimal")]
         return matches!(self, Self::BigDecimal(_));
-        #[cfg(not(feature = "with-bigdecimal"))]
-        return false;
     }
-    #[cfg(feature = "with-bigdecimal")]
     pub fn as_ref_big_decimal(&self) -> Option<&BigDecimal> {
         match self {
             Self::BigDecimal(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::BigDecimal"),
         }
     }
-    #[cfg(feature = "with-bigdecimal")]
     pub fn big_decimal_to_f64(&self) -> Option<f64> {
         use bigdecimal::ToPrimitive;
         self.as_ref_big_decimal().map(|d| d.to_f64().unwrap())
     }
-    #[cfg(not(feature = "with-bigdecimal"))]
-    pub fn as_ref_big_decimal(&self) -> Option<&bool> {
-        panic!("not Value::BigDecimal")
-    }
-    #[cfg(not(feature = "with-bigdecimal"))]
-    pub fn big_decimal_to_f64(&self) -> Option<f64> {
-        None
-    }
 }
 
+#[cfg(feature = "with-uuid")]
 impl Value {
     pub fn is_uuid(&self) -> bool {
-        #[cfg(feature = "with-uuid")]
         return matches!(self, Self::Uuid(_));
-        #[cfg(not(feature = "with-uuid"))]
-        return false;
     }
-    #[cfg(feature = "with-uuid")]
     pub fn as_ref_uuid(&self) -> Option<&Uuid> {
         match self {
             Self::Uuid(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::Uuid"),
         }
     }
-    #[cfg(not(feature = "with-uuid"))]
-    pub fn as_ref_uuid(&self) -> Option<&bool> {
-        panic!("not Value::Uuid")
-    }
 }
 
+#[cfg(feature = "postgres-array")]
 impl Value {
     pub fn is_array(&self) -> bool {
-        #[cfg(feature = "postgres-array")]
         return matches!(self, Self::Array(_));
-        #[cfg(not(feature = "postgres-array"))]
-        return false;
     }
 
-    #[cfg(feature = "postgres-array")]
     pub fn as_ref_array(&self) -> Option<&Vec<Value>> {
         match self {
             Self::Array(v) => box_to_opt_ref!(v),
             _ => panic!("not Value::Array"),
         }
     }
-    #[cfg(not(feature = "postgres-array"))]
-    pub fn as_ref_array(&self) -> Option<&bool> {
-        panic!("not Value::Array")
-    }
 }
 
 impl IntoIterator for ValueTuple {
@@ -1155,15 +1043,15 @@ where
 /// Escape a SQL string literal
 pub fn escape_string(string: &str) -> String {
     string
-        .replace("\\", "\\\\")
-        .replace("\"", "\\\"")
-        .replace("'", "\\'")
-        .replace("\0", "\\0")
-        .replace("\x08", "\\b")
-        .replace("\x09", "\\t")
-        .replace("\x1a", "\\z")
-        .replace("\n", "\\n")
-        .replace("\r", "\\r")
+        .replace('\\', "\\\\")
+        .replace('"', "\\\"")
+        .replace('\'', "\\'")
+        .replace('\0', "\\0")
+        .replace('\x08', "\\b")
+        .replace('\x09', "\\t")
+        .replace('\x1a', "\\z")
+        .replace('\n', "\\n")
+        .replace('\r', "\\r")
 }
 
 /// Unescape a SQL string literal
@@ -1273,12 +1161,9 @@ pub fn sea_value_to_json_value(value: &Value) -> Json {
         #[cfg(feature = "with-uuid")]
         Value::Uuid(Some(v)) => Json::String(v.to_string()),
         #[cfg(feature = "postgres-array")]
-        Value::Array(Some(v)) => Json::Array(
-            v.as_ref()
-                .iter()
-                .map(|v| sea_value_to_json_value(v))
-                .collect(),
-        ),
+        Value::Array(Some(v)) => {
+            Json::Array(v.as_ref().iter().map(sea_value_to_json_value).collect())
+        }
     }
 }
 
